==> dxtime_1.2.2-1.tar.gz: BUILD/src/model/dbmodel.h <==
/***************************************************************
 * Name:      dbmodel.h
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef DBMODEL_H
#define DBMODEL_H

#include <wx/event.h>
#include <wx/dynarray.h>
#include "wx/wxsqlite3.h"
#include "sqlite3.h"
#include "data/task.h"
#include "data/time.h"

WX_DECLARE_OBJARRAY(Task, dxTaskArray);
WX_DECLARE_OBJARRAY(Time, dxTimeArray);

wxDECLARE_EVENT(MODEL_UPDATED, wxCommandEvent);
wxDECLARE_EVENT(MODEL_MSG, wxCommandEvent);

#define idbmodel DbModel::instance()

class DbModel
{
public:
    DbModel();
    DbModel(wxWindow *parent);
    virtual ~DbModel();

    //database tables
    enum Tables
    {
        TB_VERSION = 0,
        TB_TASK,
        TB_TIME,
        TB_END
    };
    
    enum TimeSum
    {
        TS_TODAY = 0,
        TS_WEEK,
        TS_MONTH,
        TS_ALL
    };

    //for sort dxTimeArray
    enum TimeSort
    {
        TIS_START = 0,
        TIS_END,
        TIS_TIME,
        TIS_TASK
    };
    
    //for sort dxTaskArray
    enum TaskSort
    {
        TAS_TREE = 0
    };

    static DbModel *instance();
    void initModel();
    void shutdownModel();
    wxString getVersion();
    bool openDatabase();
    bool closeDatabase();
    int getLastId(const wxString &table);
    void fillData();

    //getters , setters etc.
    dxTaskArray tasks();
    void addTask(const Task &task);
    void editTask(const Task &task);
    Task getTask(int id);
    bool deleteTask(int id);
    bool hasTask(int id);
    bool hasSubtasks(int id);
    dxTaskArray subTasks(int id);
    void verifyTasks();
    bool treeExists(int id);
    int treeDepth(int id);
    int maxTreeDepth(int id);
    wxString taskName(int id);
    
    dxTimeArray times();
    void addTime(const Time &time);
    void editTime(const Time &time);
    Time getTime(int id);
    bool deleteTime(int id);
    bool hasTime(int id);
    wxTimeSpan timeSpanForTask(int taskid, TimeSum sum, bool subtasks);
    wxTimeSpan timeSpanForReport(int taskid, const wxDateTime &start, const wxDateTime &end, bool subtasks);
    dxTimeArray timesByTaks(int taskid, TimeSum sum, bool subtasks = false);
    dxTimeArray timesForReport(int taskid, const wxDateTime &start, const wxDateTime &end);
    int getMinutes(int id);
    wxString getTimeString(int id);
    bool timeExists(const wxDateTime &start, const wxDateTime &end, int timeid = -1);
    void setEndTime(int id, const wxDateTime &end);
    void verifyTimes();
    wxDateTime oldestDate();

    //sort functions
    void sortTask(dxTaskArray &arr, TaskSort sort, bool asc);
    void sortTime(dxTimeArray &arr, TimeSort sort, bool asc);
protected:
    void createDatabase();
    void createDatabaseTable(Tables table);
private:
    static DbModel *m_this;
    wxWindow *m_parent;
    wxSQLite3Database m_db;

    void fireUpdated();
    void fireMsg(const wxString &msg);
    void fillTasks();
    void fillTimes();
    bool hasTableId(const wxString &table, const wxString &idcolumn, int id);
    wxArrayString tableColumns(const wxString &name);
    void tryExecuteUpdate(const wxString &sql);
};

#endif  /* DBMODEL_H */

==> dxtime_1.2.4-1.tar.gz: BUILD/src/model/dbmodel.h <==
/***************************************************************
 * Name:      dbmodel.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef DBMODEL_H
#define DBMODEL_H

#include <wx/event.h>
#include <wx/dynarray.h>
#include "wx/wxsqlite3.h"
#include "sqlite3mc_amalgamation.h"
#include "data/task.h"
#include "data/time.h"

WX_DECLARE_OBJARRAY(Task, dxTaskArray);
WX_DECLARE_OBJARRAY(Time, dxTimeArray);

wxDECLARE_EVENT(MODEL_UPDATED, wxCommandEvent);
wxDECLARE_EVENT(MODEL_MSG, wxCommandEvent);

#define idbmodel DbModel::instance()

class DbModel
{
public:
    DbModel();
    DbModel(wxWindow *parent);
    virtual ~DbModel();

    //database tables
    enum Tables
    {
        TB_VERSION = 0,
        TB_TASK,
        TB_TIME,
        TB_END
    };

    enum TimeSum
    {
        TS_TODAY = 0,
        TS_WEEK,
        TS_MONTH,
        TS_YEAR,
        TS_ALL
    };

    //for sort dxTimeArray
    enum TimeSort
    {
        TIS_START = 0,
        TIS_END,
        TIS_TIME,
        TIS_TASK
    };

    //for sort dxTaskArray
    enum TaskSort
    {
        TAS_TREE = 0
    };

    static DbModel *instance();
    void initModel();
    void shutdownModel();
    wxString getVersion();
    bool openDatabase();
    bool closeDatabase();
    int getLastId(const wxString &table);
    void fillData();

    //getters , setters etc.
    dxTaskArray tasks();
    void addTask(const Task &task);
    void editTask(const Task &task);
    Task getTask(int id);
    bool deleteTask(int id);
    bool hasTask(int id);
    bool hasSubtasks(int id);
    dxTaskArray subTasks(int id);
    void verifyTasks();
    bool treeExists(int id);
    int treeDepth(int id);
    int maxTreeDepth(int id);
    wxString taskName(int id);

    dxTimeArray times();
    void addTime(const Time &time);
    void editTime(const Time &time);
    Time getTime(int id);
    bool deleteTime(int id);
    bool hasTime(int id);
    wxTimeSpan timeSpanForTask(int taskid, TimeSum sum, bool subtasks);
    wxTimeSpan timeSpanForReport(int taskid, const wxDateTime &start, const wxDateTime &end, bool subtasks);
    dxTimeArray timesByTask(int taskid, TimeSum sum, bool subtasks = false);
    dxTimeArray timesForReport(int taskid, const wxDateTime &start, const wxDateTime &end);
    int getMinutes(int id);
    wxString getTimeString(int id);
    bool timeExists(const wxDateTime &start, const wxDateTime &end, int timeid = -1);
    void setEndTime(int id, const wxDateTime &end);
    void verifyTimes();
    wxDateTime oldestDate();

    //sort functions
    void sortTask(dxTaskArray &arr, TaskSort sort, bool asc);
    void sortTime(dxTimeArray &arr, TimeSort sort, bool asc);
protected:
    void createDatabase();
    void createDatabaseTable(Tables table);
private:
    static DbModel *m_this;
    wxWindow *m_parent;
    wxSQLite3Database m_db;

    void fireUpdated();
    void fireMsg(const wxString &msg);
    void fillTasks();
    void fillTimes();
    bool hasTableId(const wxString &table, const wxString &idcolumn, int id);
    wxArrayString tableColumns(const wxString &name);
    void tryExecuteUpdate(const wxString &sql);
};

#endif  /* DBMODEL_H */
